@@ -1,10 +1,14 @@
+from __future__ import print_function
+from mongoalchemy.py3compat import *
+
 from itertools import chain
 from bson.objectid import ObjectId
 from abc import ABCMeta, abstractmethod
 
+@add_metaclass(ABCMeta)
 class Operation(object):
     __metaclass__ = ABCMeta
-    
+
     execute = abstractmethod(lambda self : None)
 
     def update_cache(self): pass
@@ -41,7 +45,7 @@ class UpdateDocumentOp(Operation):
         self.dirty_ops = document.get_dirty_ops(with_required=upsert)
         for key, op in chain(update_ops.items(), kwargs.items()):
             key = str(key)
-            for current_op, keys in self.dirty_ops.items():
+            for current_op, keys in list(self.dirty_ops.items()):
                 if key not in keys:
                     continue
                 self.dirty_ops.setdefault(op,{})[key] = keys[key]
@@ -64,9 +68,9 @@ class UpdateOp(Operation):
         self.update_data = update_obj.update_data
         self.upsert = update_obj._get_upsert()
         self.multi = update_obj._get_multi()
-        
+
     def execute(self):
-        return self.collection.update(self.query, self.update_data, multi=self.multi, 
+        return self.collection.update(self.query, self.update_data, multi=self.multi,
                                upsert=self.upsert, safe=self.safe)
 
 
@@ -99,7 +103,7 @@ class RemoveOp(Operation):
         self.ensure_indexes()
         return self.collection.remove(self.query, safe=self.safe)
 
-        
+
 class RemoveDocumentOp(Operation):
     def __init__(self, trans_id, session, obj, safe):
         self.trans_id = trans_id
@@ -118,4 +122,3 @@ class RemoveDocumentOp(Operation):
 
         collection = db[self.type.get_collection_name()]
         return collection.remove(self.id, safe=self.safe)
-
